searchData={"content_type":"text/markdown","items":[{"doc":"Retrieval of configuration settings from external sources, such as OS env vars.\n\nThis module is an alternative to app env for retrieval of configuration settings. It allows you\nto properly consolidate system settings, define per-env defaults, add strong typing, and\ncompile time guarantees.","ref":"Provider.html","title":"Provider","type":"module"},{"doc":"defmodule MySystem.Config do\n      use Provider,\n        source: Provider.SystemEnv,\n        params: [\n          {:db_host, dev: \"localhost\"},\n          {:db_name, dev: \"my_db_dev\", test: \"my_db_test\"},\n          {:db_pool_size, type: :integer, default: 10},\n          # ...\n        ]\n    end\n\nThis will generate the following functions in the module:\n\n  - `load!` - validates that all parameters are correctly provided and stores them in the cache\n  - `db_host`, `db_name`, `db_pool_size`, ... - getter of each declared parameter","ref":"Provider.html#module-basic-example","title":"Basic example - Provider","type":"module"},{"doc":"Each param is described in the shape of `{param_name, param_spec}`, where `param_name` is an\natom, and `param_spec` is a keyword list. Providing only `param_name` (without a tuple), is the\nsame as `{param_name, []}`.\n\nThe following keys can be used in the `param_spec`:\n\n- `:type` - Param type (see `t:type/0`). Defaults to `:string`.\n- `:default` - Default value used if the param is not provided. Defaults to `nil` (no default).\n- `:dev` - Default value in `:dev` and `:test` mix env. Defaults to `nil` (no default).\n- `:test` - Default value in `:test` mix env. Defaults to `nil` (no default).\n\nDefault options are considered in the following order:\n\n1. `:test` (if mix env is `:test`)\n2. `:dev` (if mix env is either `:dev` or `:test`)\n3. `:default`\n\nFor example, if `:test` and `:default` options are given, the `:test` value will be used as a\ndefault in `:test` env, while `:default` will be used in all other envs.\n\nWhen you invoke the generated functions, values will be retrieved from the external storage\n(e.g. OS env). If some value is not available, a default value will be used (if provided). The\nvalues are then casted according to the parameter type.\n\nEach default can be a constant, but it can also be an expression, which is evaluated at runtime.\nFor example:\n\n    defmodule MySystem.Config do\n      use Provider,\n        source: Provider.SystemEnv,\n        params: [\n          # db_name/0 will be invoked when you try to retrieve this parameter (or all parameters)\n          {:db_name, dev: db_name()},\n          # ...\n        ]\n\n      defp db_name(), do: #...\n    end\n\nIt's worth noting that `Provider` performs compile-time purging of needless defaults. When you\ncompile the code in `:prod`, `:dev` and `:test` defaults will not be included in the binaries.\nConsequently, any private function invoked only in dev/test will also not be invoked, and you'll\nget a compiler warning when compiling the code in prod. To eliminate such warnings, you can\nconditionally define the function only in required mix env, by moving the function definition\nunder an `if Mix.env() == ` conditional.","ref":"Provider.html#module-describing-params","title":"Describing params - Provider","type":"module"},{"doc":"The config module will contain the `template/0` function which generates the configuration\ntemplate. To print the template to stdout, you can invoke:\n\n    MIX_ENV=prod mix compile\n    MIX_ENV=prod mix run --no-start -e 'IO.puts(MySystem.Config.template())'","ref":"Provider.html#module-generating-template","title":"Generating template - Provider","type":"module"},{"doc":"The basic retrieval functionality is available via functions of this module, such as\n`fetch_all/2`, or `fetch_one/2`. These functions are a lower level plumbing API which is less\nconvenient to use, but more flexible. Most of the time the `use`-based interface will serve you\nbetter, but if you have more specific needs which are not covered by that, you can reach for\nthese functions.","ref":"Provider.html#module-lower-level-api","title":"Lower level API - Provider","type":"module"},{"doc":"Retrieves all params according to the given specification.","ref":"Provider.html#fetch_all/2","title":"Provider.fetch_all/2","type":"function"},{"doc":"","ref":"Provider.html#t:data/0","title":"Provider.data/0","type":"type"},{"doc":"","ref":"Provider.html#t:param_name/0","title":"Provider.param_name/0","type":"type"},{"doc":"","ref":"Provider.html#t:param_spec/0","title":"Provider.param_spec/0","type":"type"},{"doc":"","ref":"Provider.html#t:params/0","title":"Provider.params/0","type":"type"},{"doc":"","ref":"Provider.html#t:source/0","title":"Provider.source/0","type":"type"},{"doc":"","ref":"Provider.html#t:type/0","title":"Provider.type/0","type":"type"},{"doc":"","ref":"Provider.html#t:value/0","title":"Provider.value/0","type":"type"},{"doc":"Contract for storage sources.","ref":"Provider.Source.html","title":"Provider.Source","type":"behaviour"},{"doc":"Invoked to convert the param name to storage specific name.","ref":"Provider.Source.html#c:display_name/1","title":"Provider.Source.display_name/1","type":"callback"},{"doc":"Invoked to create operator template.","ref":"Provider.Source.html#c:template/1","title":"Provider.Source.template/1","type":"callback"},{"doc":"Invoked to provide the values for the given parameters.\n\nThis function should return all values in the requested orders. For each param which is not\navailable, `nil` should be returned.","ref":"Provider.Source.html#c:values/1","title":"Provider.Source.values/1","type":"callback"},{"doc":"Provider source which retrieves values from OS env vars.","ref":"Provider.SystemEnv.html","title":"Provider.SystemEnv","type":"module"},{"doc":"# Provider\n\n[![hex.pm](https://img.shields.io/hexpm/v/provider.svg?style=flat-square)](https://hex.pm/packages/provider)\n[![hexdocs.pm](https://img.shields.io/badge/docs-latest-green.svg?style=flat-square)](https://hexdocs.pm/provider/)\n![Build Status](https://github.com/sasa1977/provider/workflows/provider/badge.svg)\n\nProvider is a library which helps managing the operator configuration of a system.\n\nThe term operator configuration refers to the settings which have to be provided by the operator of the system before the system is started. Typical examples include database credentials (host, database name, login, password), http and https ports, 3rd party API keys, and such. On the other hand, things such as Ecto database adapter, or dev/test-only variations such as mocks, are not a part of operator configuration.\n\nProvider offers the following features:\n\n- Consolidation of operator settings into a single place\n- Basic strong typing (automatic conversion of external settings to string, integer, or boolean)\n- Compile time guarantees\n- Runtime validation during the system startup\n- Automatic creation of operator templates, such as .env file which lists all configuration parameters\n- In-code provisioning of dev/test defaults, which removes the need to use external files such as .envrc, or manage duplicate settings in CI yamls\n\nProvider was born out of real need and is based on lessons learned and issues experienced with config scripts and app config. Provider doesn't use app config, and is instead only focused on fetching values from external sources, such as OS env. The responsibility of working with those values is left to the client.\n\nProvider is currently used in a few small projects, but it's not exhaustively tested nor optimized. The API is not considered as stable, and it may change significantly. Provider currently only supports the features needed by the projects it powers. Therefore, it has a couple of limitations:\n\n- Provider can't be used to provide compile-time settings or to configure applications which require the settings before the main app is started (such as Kernel or Logger). In these cases you'll still need to use config.exs & friends.\n- Only OS env is currently supported as an external source.\n- No support for `nil` or empty strings. Each configured value has to be provided, and it can't be an empty string.\n\nTackling these shortcomings is on the roadmap, but it hasn't happen yet, because there was no actual need to address them so far.","ref":"readme.html","title":"Provider","type":"extras"},{"doc":"Add provider as a dependency:\n\n```elixir\ndefmodule MySystem.MixProject do\n  # ...\n\n  defp deps do\n    [\n      {:provider, \"~> 0.1\"},\n      # ...\n    ]\n  end\nend\n```\n\nCreate a module where you'll consolidate your configuration:\n\n```elixir\ndefmodule MySystem.Config do\n  use Provider,\n    source: Provider.SystemEnv,\n    params: [\n      # The `:dev` option sets a dev/test default. This default won't be used in `:prod` mix env.\n      {:db_host, dev: \"localhost\"},\n\n      # The `:test` option overrides the default in the test mix env.\n      {:db_name, dev: \"my_db_dev\", test: \"my_db_test\"},\n\n      # The `:default` option sets a default in all mix envs, which means that this setting is\n      # optional.\n      #\n      # Owing to `type: :integer`, the external input will be converted into an integer. An error\n      # will be raised if this conversion fails.\n      {:db_pool_size, type: :integer, default: 10},\n\n      # ...\n    ]\nend\n```\n\nValidate configuration during app startup:\n\n```elixir\ndefmodule MySystem.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      MySystem.Config\n    ]\n\n    # ...\n  end\n\n  # ...\nend\n\n```\n\nUse config module functions to fetch the values:\n\n```elixir\ndefmodule MySystem.Repo do\n  use Ecto.Repo,\n    otp_app: :my_system,\n    adapter: Ecto.Adapters.Postgres\n\n  @impl Ecto.Repo\n  def init(_type, config) do\n    {:ok,\n      Keyword.merge(\n        config,\n        hostname: MySystem.Config.db_host(),\n        database_name: MySystem.Config.db_name(),\n        pool_size: MySystem.Config.db_pool_size(),\n        # ...\n      )\n    }\n  end\nend\n```\n\nInvoke `MySystem.Config.template()` in prod Mix env to generate a .env template with all configuration parameters. The function will return a string which you can print to screen or save to a file.","ref":"readme.html#quick-start","title":"Quick start - Provider","type":"extras"},{"doc":"There are a couple of other libraries available with similar features:\n\n- [Skogsrå](https://github.com/gmtprime/skogsra)\n- [Specify](https://github.com/Qqwy/elixir-specify)\n- [Vapor](https://github.com/keathley/vapor)","ref":"readme.html#alternatives","title":"Alternatives - Provider","type":"extras"},{"doc":"[MIT](LICENSE)","ref":"readme.html#license","title":"License - Provider","type":"extras"},{"doc":"Copyright\n  2020-2023, [Very Big Things](https://verybigthings.com/)\n  2023-      Saša Jurić\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the \"Software\"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"LICENSE","type":"extras"},{"doc":"# Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"- First version","ref":"changelog.html#0-1-0","title":"0.1.0 - Changelog","type":"extras"}]}